using System;
using System.Threading;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Contexts;

[Synchronization(SynchronizationAttribute.REQUIRED)]
class Counter : ContextBoundObject
{
	private long counter = 0;
	public void IncrementCounter()
	{
		Console.WriteLine("Thread {0} enters IncrementCounter in Context {1}", Thread.CurrentThread.GetHashCode(), Thread.CurrentContext.ContextID);
		long oldValue = counter;
		oldValue += 1;
		Thread.Sleep(0);
		counter = oldValue;
		Console.WriteLine("Thread {0} leaves IncrementCounter in Context {1}", Thread.CurrentThread.GetHashCode(), Thread.CurrentContext.ContextID);
	}

	public long GetCounter()
	{
		return counter;
	}

}

class Action
{
	private Counter counter;

	public Action (int start, int increment, int end, Counter counter)
	{
		this.start = start;
		this.increment = increment;
		this.end = end;
		this.counter = counter;

		Console.WriteLine("Is counter a proxy? {0}", RemotingServices.IsTransparentProxy(counter));

	}

	private int start;
	private int increment;
	private int end;

	public void Count()
	{
		Console.WriteLine("Thread {0} enters Count in Context {1}", Thread.CurrentThread.GetHashCode(), Thread.CurrentContext.ContextID);
		for (int i = start; i <= end; i += increment)
		{
			Console.WriteLine("Thread {0} in Context {1} has value {1}", Thread.CurrentThread.GetHashCode(), Thread.CurrentContext.ContextID, i);
			counter.IncrementCounter();	
		}
		Console.WriteLine("Thread {0} exits Count in Context {1}", Thread.CurrentThread.GetHashCode(), Thread.CurrentContext.ContextID);
	}

}

class Example
{
	static void Main()
	{
		Console.WriteLine("Program starts in Context {0}", Thread.CurrentContext.ContextID);

		Counter counter = new Counter();

		Action a1 = new Action(1, 10, 50, counter);
		Action a2 = new Action(100, 10, 150, counter);

		Console.WriteLine("Is a1 a proxy? {0}", RemotingServices.IsTransparentProxy(a1));
		Console.WriteLine("Is a2 a proxy? {0}", RemotingServices.IsTransparentProxy(a2));

		ThreadStart ts1 = new ThreadStart(a1.Count);
		ThreadStart ts2 = new ThreadStart(a2.Count);

		Thread t1 = new Thread(ts1);
		Thread t2 = new Thread(ts2);

		t1.Start();
		t2.Start();

		t1.Join();
		t2.Join();

		Console.WriteLine("Number of counts: {0}", counter.GetCounter());

	}
}

