// ClrHost.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#import <mscorlib.tlb> raw_interfaces_only
using namespace mscorlib;

void ReportErrorMessage(HRESULT hr)
{
	LPVOID lpMsgBuf;
	DWORD bOk = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL, hr, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0, NULL); 
	if (bOk)
	{
		_ftprintf(stderr, _T("%s\n"), (LPTSTR)((LPTSTR)lpMsgBuf));
		LocalFree( lpMsgBuf );
	}
	else
		_ftprintf(stderr, _T("Unknown Error\n"));
}

int main(int argc, char* argv[])
{
	CComBSTR bstrVer(L"v0.0.0000");
	CComBSTR bstrCLRType(L"wks");
	CComPtr<ICorRuntimeHost> pHost;

	HRESULT hr = CorBindToRuntimeEx(NULL, bstrCLRType,    
                      STARTUP_LOADER_OPTIMIZATION_MULTI_DOMAIN | STARTUP_CONCURRENT_GC, 
                      CLSID_CorRuntimeHost, IID_ICorRuntimeHost, (void **)&pHost);
	if (!SUCCEEDED(hr))
	{
		printf("CorBindToRuntime failed. ");
		ReportErrorMessage(hr);
		return 1;
	}

	WCHAR buf[30];
	DWORD dwLength = 0;
	hr = GetCORVersion(buf, 30, &dwLength);
	wprintf(L"Version %s of the CLR loaded...\n", buf);
	
	pHost->Start();

	CComPtr<IUnknown> punkDefaultDomain;
	hr = pHost->GetDefaultDomain(&punkDefaultDomain);
	if (!SUCCEEDED(hr))
	{
		printf("GetDefaultDomain failed. ");
		ReportErrorMessage(hr);
		return 1;
	}	
	 
	CComPtr<_AppDomain> pDefaultDomain;
	hr = punkDefaultDomain->QueryInterface(__uuidof(_AppDomain), (void**) &pDefaultDomain);
	if (!SUCCEEDED(hr))
	{
		printf("Query Interface failed. ");
		ReportErrorMessage(hr);
		return 1;
	}

	printf("Default Domain started...\n");

	CComBSTR bstrAssembly(L"AppHost");
	CComBSTR bstrType(L"ManagedHost");
    _ObjectHandle* pObjectHandle = NULL;
	hr = pDefaultDomain->CreateInstance(bstrAssembly, bstrType, &pObjectHandle);
	if (!SUCCEEDED(hr))
	{
		printf("Type creation failed. ");
		ReportErrorMessage(hr);
		return 1;
	}

	VARIANT v;
	VariantInit(&v);
	hr = pObjectHandle->Unwrap(&v);
	if (!SUCCEEDED(hr))
	{
		printf("Unwrap failed. ");
		ReportErrorMessage(hr);
		return 1;
	}

	_Object* pobj;
	hr = v.pdispVal->QueryInterface(__uuidof(_Object), (void**) &pobj);
	if (!SUCCEEDED(hr))
	{
		printf("Object QI failed. ");
		ReportErrorMessage(hr);
		return 1;
	}

	_Type* pType;
	hr = pobj->GetType(&pType);
	if (!SUCCEEDED(hr))
	{
		printf("GetType failed. ");
		ReportErrorMessage(hr);
		return 1;
	}

	CComBSTR bstrArgument(L"StartManagedHost");
	_MethodInfo* mi;
	hr = pType->GetMethod_6(bstrArgument, &mi);
	if (!SUCCEEDED(hr))
	{
		printf("GetMethod failed. ");
		ReportErrorMessage(hr);
		return 1;
	}

	if (mi == NULL)
	{
		printf("MethodInfo is NULL.\n");
		return 1;
	}

	CComVariant vHandle;
	hr = pObjectHandle->Unwrap(&vHandle);
	if (!SUCCEEDED(hr))
	{
		printf("Unwrap failed. ");
		ReportErrorMessage(hr);
		return 1;
	}

	CComVariant vReturnValue;
	hr = mi->Invoke_3(vHandle, NULL, &vReturnValue);
	if (!SUCCEEDED(hr))
	{
		printf("Invoke failed. ");
		ReportErrorMessage(hr);
		return 1;
	};

	pHost->Stop();
	return 0;
}

